from datetime import datetime

from tic.shared.dto import Dto
from tic.shared.apc.enums.apc_status import ApcStatus
from tic.shared.apc.enums.apc_category import ApcCategory
from tic.shared.apc.enums.apc_verdict import ApcVerdict
from tic.shared.utils.date_util import (
    epoch_time_to_utc_datetime,
    utc_datetime_to_epoch_time,
    utc_now,
)
from tic.shared.apc.utils import get_apc_description, get_apc_verdict


class Apc(Dto):
    encode_names = {"_verdict": "verdict"}
    decode_names = {"verdict": "_verdict"}
    decode_settings = {
        "status": lambda x: None if x is None else ApcStatus(x),
        "category": lambda x: None if x is None else ApcCategory(x),
        "_verdict": lambda x: None if x is None else ApcVerdict(x),
        "ttl": lambda x: epoch_time_to_utc_datetime(utc_now())
        if x is None
        else epoch_time_to_utc_datetime(x),
        "query_date": lambda x: epoch_time_to_utc_datetime(utc_now())
        if x is None
        else epoch_time_to_utc_datetime(x),
    }

    encode_settings = {
        "status": lambda x: None if x is None else x.value,
        "category": lambda x: None if x is None else x.value,
        "_verdict": lambda x: None if x is None else x.value,
        "ttl": lambda x: utc_datetime_to_epoch_time(utc_now())
        if x is None
        else utc_datetime_to_epoch_time(x),
        "query_date": lambda x: utc_datetime_to_epoch_time(utc_now())
        if x is None
        else utc_datetime_to_epoch_time(x),
    }

    def __init__(
        self,
        stage_version: float = 0.0,  # the stage version of whatever stage produces the apc result (APC_LOOKUP*)
        ttl: datetime = None,  # Absolute timestamp at which this APC data expires.
        status: ApcStatus = None,
        category: ApcCategory = None,
        verdict: ApcVerdict = None,
        query_date: datetime = None,
    ):
        self.stage_version = stage_version
        self.ttl = ttl if ttl is not None else utc_now()
        self.status = status
        self.category = category
        self._verdict: ApcVerdict = (
            verdict
            if verdict is not None
            else (
                get_apc_verdict(get_apc_description(category))
                if category is not None
                else None
            )
        )
        self.query_date = query_date

    @property
    def verdict(self) -> ApcVerdict:
        if self._verdict is None and self.category is not None:
            self._verdict = get_apc_verdict(get_apc_description(self.category))
        return self._verdict

    @verdict.setter
    def verdict(self, value: ApcVerdict):
        self._verdict = value
